/*	
	Client application (distributed failure detection)
	for benchmarking

	Pedro Martins <pmartins@di.fc.ul.pt>	

--[TTCB 1.11]----------------------------------------------------

    (Trusted) Timely Computing Base 
    (C) 1999-2003, Navigators Group 
    		   http://www.navigators.di.fc.ul.pt
    
-----------------------------------------------------------------	
*/

#include "tcb/system_defs.h"

#ifdef DIST_FAULT

#include "tcb/tcb.h"
#include <signal.h>
#include <sched.h>
#include <netinet/in.h>
#include <linux/inet.h>

char child_stack[1000];
char child_stack_end;

int thrd1;

char p[4324];

descriptor_t tcbDesc;
char ip_addr[3000];

int number_of_machines;
char ttcb_id;

#define TCB_PORT		41

void exit_on_error(char * msg)
{
	if (errno==0)
	 fprintf(stderr,"%s",msg);
	  else
	   perror(msg);
	
	printf("\n");
   
	kill(thrd1,SIGINT);   
	closeTCBConnection(&tcbDesc);   
	exit(0);   
}

void sigint_handler(int par)
{
	kill(thrd1,SIGINT);    	
	closeTCBConnection(&tcbDesc);   
	exit(0);
}

int fn(void* arg)
{
	fail_info_to_app fi;
	
	while (1)
	{
	  TCBwaitInfo(&tcbDesc,&fi);  	
	}
	return 0;
}
#endif

int main(int argc,char* argv[])
{
#ifdef DIST_FAULT
	int res;	
	int ret;	
	TCBtimestamp sndtime,gsndtime,rcv_evt;
	unsigned int tag;
	TCBtimeinterval delay,error;	
	Func_Id p;
	int one;
 	struct sockaddr_in cnt,serv;
 	int len; 	
 	char buffer[sizeof(distAction_id_t)];
 	distAction_id_t * distAction;
	int t,a;	
	unsigned char mask=0xFF;
        unsigned char andmask=1; 
        tcb_kernel_info_t krn_info;	
	        	
	if ((ret=openTCBConnection(&tcbDesc))<OK)
	{
		printTCBerror(ret);
		printf("On opening a connection...\n");
		exit(1);
	}

	// get local TTCB id	
	ttcb_id=getTCBId(&tcbDesc);

	if (ttcb_id < 0){
		closeTCBConnection(&tcbDesc);
		printTCBerror(ret);
		printf("On getting the TTCB id...\n");
		exit(1);
	}
	

	// gather TTCB kernel info
	ret = getTCBKernelInfo(&tcbDesc, &krn_info);

	if (ret < 0){
		closeTCBConnection(&tcbDesc);
		printTCBerror(ret);
		printf("On getting TTCB kernel info...\n");
		exit(1);
	}

	number_of_machines = krn_info.number_of_system_machines;
	
	if (number_of_machines!=1)
	{
	
	for (a=0;a<ttcb_id;a++)
	 andmask<<=1;		
		
	 andmask=~andmask;
	 mask=mask & andmask;		
	}
	
	//create a thread to read WaitInfo data  	
	thrd1=clone(&fn,&child_stack_end,CLONE_VM,(void*)0);

	signal(SIGINT,sigint_handler);
	
 	strcpy(p.module,"app_module"); 
 	strcpy(p.funcname,"timelyExecDFD"); 
 	strcpy(p.wcetname,"timelyExec_wcet");  
 	
 	while (1)
 	{
 	 ret = getTCBLocalAndGlobalTimestamp(&tcbDesc, &sndtime, &gsndtime); 	 
 	 if (ret < 0){
		exit_on_error("On getting a global timestamp from the TTCB...\n");
	 }	
 	 
 	 distAction=(distAction_id_t*)&buffer;
 	 ret = startTCBDistFD(&tcbDesc,mask,TCB_PORT,sndtime,gsndtime,1000000,0,p,distAction);
 
 	 if (ret<0)
  	  {	
  	  	if ((ret!=ALREADY_FAILED)&&(ret!=BURSTY_SENDS)&&(ret!=TOO_MANY_DISTRIBUTED_FD)){
  			printf("%d\n",ret);
  			exit_on_error("On starting a distributed FD operation...\n");
			exit(1);
  		} 
  	  }  	
 	}
#else 
	while (1){};
#endif 
}

