/*	
	Client application (local failure detection)
	for benchmarking

	Pedro Martins <pmartins@di.fc.ul.pt>

--[TTCB 1.11]----------------------------------------------------

    (Trusted) Timely Computing Base 
    (C) 1999-2003, Navigators Group 
    		   http://www.navigators.di.fc.ul.pt
    	
-----------------------------------------------------------------	
*/

#include "tcb/system_defs.h"

#ifdef LOCAL_FAULT

#include "tcb/tcb.h"
#include <sys/time.h>
#include <sys/signal.h>

descriptor_t tcbDesc;

void exit_on_error(char * msg)
{
	if (errno==0)
	 fprintf(stderr,"%s",msg);
	  else
	   perror(msg);
	   
	closeTCBConnection(&tcbDesc);   
	exit(0);   
}

void sigint_handler(int par)
{
	
	closeTCBConnection(&tcbDesc);   
	exit(0);
}
#endif

int main(int argc,char* argv[])
{
#ifdef LOCAL_FAULT
	int res,desc_lfd,ret;
	TCBtimestamp start_evt,end_evt;
	TCBtimeinterval proc_delay;
	Func_Id funcId;
	char faulty;
	unsigned long long estimated=0;
	
	signal(SIGINT,sigint_handler);	
	
	// open a conection to TCB
	if ( (ret=openTCBConnection(&tcbDesc))<OK)
	{
		printTCBerror(ret);
		printf("On opening a connection...\n");
		exit(1);
	}

	
	strcpy(funcId.module,"app_module");
	strcpy(funcId.funcname,"timelyExecLFD");
	strcpy(funcId.wcetname,"timelyExec_wcet");
	
	while (1)
	{
	ret = getTCBTimestamp(&tcbDesc, &start_evt);
	
	if (ret < 0){
		closeTCBConnection(&tcbDesc);
		printTCBerror(ret);
		printf("On getting a local timestamp...\n");
		exit(1);
	}

	desc_lfd = startTCBLocalFD(&tcbDesc,start_evt,10000000,0,funcId);

	if ((desc_lfd<0)&&(desc_lfd!=ALREADY_FAILED))
	{	
		closeTCBConnection(&tcbDesc);
		printTCBerror(desc_lfd);
		printf("On starting a local failure detection operation...\n");
		exit(1);
	}	
	usleep(20000);
	
  	ret = endTCBLocalFD(&tcbDesc,desc_lfd,&end_evt,&proc_delay,&faulty);
  	if (ret<0)
	{	
		closeTCBConnection(&tcbDesc);
		printTCBerror(ret);
		printf("On finishing a local failure detection operation...\n");
		exit(1);	
	}		
	}
	
	return 0;
#else
	while (1){};
#endif
}
