/*
	(TTCB)
	Distributed Measurement Demo Application	

	Pedro Martins <pmartins@di.fc.ul.pt>
	
--[TTCB 1.11]----------------------------------------------------

    (Trusted) Timely Computing Base
    (C) 1999-2003, Navigators Group 
    		   http://www.navigators.di.fc.ul.pt
    
-----------------------------------------------------------------	
*/

#include "tcb/tcb.h"
#include "tcb/system_defs.h"
#include <signal.h>
#include "defs.h"
#include <netinet/in.h>
#include <linux/inet.h>

descriptor_t tcbDesc;
char ip_addr[3000];
int sock_fd;	

void exit_on_error(char * msg, int code)
{
	printf("%s ",msg);

	if (errno==0)
	 printTCBerror(code);
	  else
	   perror(msg);
	
	printf("\n");
	   
	closeTCBConnection(&tcbDesc);   
	close(sock_fd);
	exit(1);   
}

void sigint_handler(int par)
{
	closeTCBConnection(&tcbDesc);   
	printf("\nTCBPing stopped by user\n\n");
	close(sock_fd);
	exit(0);
}


int main(int argc,char* argv[])
{
	int ret;
	TCBtimestamp sndtime, rcv_evt , grcv_evt;
	TCBtimeinterval delay;	
 	struct sockaddr_in cnt,serv;
 	int len; 	
 	char buffer[1000];
 	int one=1;
	unsigned long long cnts;
	pingResp* pt;
	distAction_id_t *distAction;
	fd_set fds;
	struct timeval tv;

	printf("\n----------------------------------------------------------\n");
	printf("TCBping (Client) - a linux like ping process using the TCB\n");
	printf("----------------------------------------------------------\n\n");

	bzero(&tcbDesc,sizeof(descriptor_t));		        		
	        	
	if ((ret = openTCBConnection(&tcbDesc))<OK)
	{
		printTCBerror(ret);
		printf("On opening a connection...\n");
		exit(1);
	}
	
	signal(SIGINT,sigint_handler);
	
	if ((sock_fd=socket(AF_INET,SOCK_DGRAM,0))<0)
	{
		exit_on_error("Error on creating an UDP socket...\n\n",sock_fd);
	}

	setsockopt(sock_fd,SOL_SOCKET,SO_REUSEADDR|SO_BROADCAST,&one,sizeof(one));
	
 	bzero((char*)&cnt,0,sizeof(cnt));
	
 	cnt.sin_family=AF_INET;
 	cnt.sin_addr.s_addr=htonl(INADDR_ANY);
 	cnt.sin_port=htons(0);
 	
 	if ((ret=bind(sock_fd,(struct sockaddr*)&cnt,sizeof(cnt)))!=0)
 	 {	
 	 	exit_on_error("Error on binding an UDP socket...\n",ret);
 	 }
 	
 	bzero((char*)&serv,0,sizeof(serv));
 	
 	serv.sin_family=AF_INET;
 	serv.sin_addr.s_addr=inet_addr(BROADCAST_ADDR);
 	serv.sin_port=htons(16006);
 
 	cnts=0;
 	while (1)
 	{
 	cnts++;
 	if ((cnts>400000))
 	{
 	 ret = getTCBGlobalTimestamp(&tcbDesc, &sndtime); 	 
 	 if (ret < 0){
		exit_on_error("On getting a global timestamp from the TCB...\n",ret);
	 }

	distAction=(distAction_id_t*)&buffer;
 	ret = startTCBDistMeasurement(&tcbDesc,sndtime,distAction);
 	if (ret < 0){
		exit_on_error("On starting a distributed measurement...\n",ret);
	 }

  	 if ((ret=sendto(sock_fd,(char*)buffer,sizeof(distAction_id_t),0,(struct sockaddr*)&serv,sizeof(serv)))==-1)
  	  {
  	  	exit_on_error("On sending a message to an UDP socket...\n",ret);
  	  }
  	 cnts=0;
  	}
 	
  	//Non Blocking reading
  	FD_ZERO(&fds);
	FD_SET(sock_fd,&fds);	 		 	
	 	
	ret=1;	
	while (ret){	

	tv.tv_sec=0;
	tv.tv_usec=0;
	 	
	ret=select(sock_fd+1,&fds,NULL,NULL,&tv);
	 	
	if (ret)
	{
 	len=sizeof(cnt);
 	if ((ret=recvfrom(sock_fd,(char*)buffer,sizeof(pingResp)+sizeof(distAction_id_t),0,(struct sockaddr*)&cnt,&len))!=-1) 	
 	{
 	 distAction = (distAction_id_t*)&buffer; 	
 	 if (!TFD_ENABLED(distAction))
 	 	ret = endTCBDistMeasurement(&tcbDesc,distAction,&rcv_evt,&grcv_evt,&delay);
 	 else {
		closeTCBConnection(&tcbDesc);
		printTCBerror(ret);
		printf("TFD operation not expected...\n");
		exit(1);	
	  }

	 if (ret < 0){
		exit_on_error("On ending a distributed measurement...\n",ret);
	 } 
 	 pt=(pingResp*)(buffer+sizeof(distAction_id_t));
 	 printf("------------------------------------\n");	
 	 printf("Packet From: %s\n",inet_ntoa(cnt.sin_addr.s_addr));
 	 printf("------------------------------------\n");
 	 printf("Send   : delay <%llu> us\n",pt->delay/1000);
 	 printf("Receive: delay <%llu> us\n",delay/1000);
 	 printf("\n");
 	}	  
 	 else
 	 if (ret<0)  
 	  {
 	     exit_on_error("On getting a message from an UDP socket...\n",ret);
 	  }
 	}  //  !retval
 	}  // while ret
 
 	}  //  while
}

