/*
	(TTCB)
	Distributed Measurement Demo Application	

	Pedro Martins <pmartins@di.fc.ul.pt>
	
--[TTCB 1.11]----------------------------------------------------

    (Trusted) Timely Computing Base 
    (C) 1999-2003, Navigators Group 
    		   http://www.navigators.di.fc.ul.pt
    
-----------------------------------------------------------------	
*/

#include "tcb/tcb.h"
#include <signal.h>
#include "defs.h"
#include <netinet/in.h>
#include <linux/inet.h>

descriptor_t tcbDesc;
int sock_fd;

void exit_on_error(char * msg,int code)
{
	printf("%s ",msg);

	if (errno==0)
	 printTCBerror(code);
	  else
	   perror(msg);
	
	printf("\n");
	   
	closeTCBConnection(&tcbDesc);   
	close(sock_fd);
	exit(1);   
}

void sigint_handler(int par)
{
	closeTCBConnection(&tcbDesc);   
	printf("\nTCBPing Server stopped by user\n\n");
	close(sock_fd);
	exit(0);
}

int main()
{
	int ret;	
	char ip_addr[300];
	TCBtimestamp sndtime,rcv_evt,grcv_evt;
	TCBtimeinterval delay,delay_rnd;
 	struct sockaddr_in serv,cnt;
 	int len; 	
 	int one=1;
 	char buffer[1000];
	pingResp* pt;
	distAction_id_t *distAction;

	printf("\n----------------------------------------------------------\n");
	printf("TCBping (Server) - a linux like ping process using the TCB\n");
	printf("----------------------------------------------------------\n");
	
	bzero(&tcbDesc,sizeof(descriptor_t));

	if ( (ret=openTCBConnection(&tcbDesc))<OK)
	{
		printTCBerror(ret);
		printf("On opening a connection...\n");
		exit(1);
	}
	
	signal(SIGINT,sigint_handler);
	
	if ((sock_fd=socket(AF_INET,SOCK_DGRAM,0))<0)
	{
		exit_on_error("Error on creating an UDP socket...\n\n",sock_fd);
	}

	setsockopt(sock_fd,SOL_SOCKET,SO_REUSEADDR|SO_BROADCAST,&one,sizeof(one));

 	bzero((char*)&serv,0,sizeof(cnt));
 	
 	serv.sin_family=AF_INET;
 	serv.sin_addr.s_addr=htonl(INADDR_ANY);
 	serv.sin_port=htons(16006); 	
 	
 	if ((ret=bind(sock_fd,(struct sockaddr*)&serv,sizeof(serv)))!=0)
 	 {	
 	 	exit_on_error("Error on binding an UDP socket...\n",ret);
 	 }
 
	while (1)
	{ 	
	 	
 	len=sizeof(serv); 	

	if ((ret=recvfrom(sock_fd,(char*)buffer,sizeof(distAction_id_t),0,(struct sockaddr*)&cnt,&len))!=-1) 	 	 	 
 	{
 	  distAction = (distAction_id_t*)&buffer; 		
 	  if (!TFD_ENABLED(distAction))
	 	  ret = endTCBDistMeasurement(&tcbDesc,distAction,&rcv_evt,&grcv_evt,&delay);
	  else {
		closeTCBConnection(&tcbDesc);
		printTCBerror(ret);
		printf("TFD operation not expected...\n");
		exit(1);	
	  }

	  if (ret < 0){
		exit_on_error("On ending a distributed measurement...\n",ret);
	  }	
 	  pt=(pingResp*)(buffer+sizeof(distAction_id_t));	
 	  pt->delay=delay;	
 
 	  ret = getTCBGlobalTimestamp(&tcbDesc, &sndtime); 	  	
 
 	  if (ret < 0){
		exit_on_error("On getting a global timestamp from the TCB...\n",ret);
	  }

	  distAction=(distAction_id_t*)&buffer;
 	  ret = startTCBDistMeasurement(&tcbDesc,sndtime,distAction);
 	  if (ret < 0){
		exit_on_error("On starting a distributed measurement...\n",ret);
	   }	
 
	  if ((ret=sendto(sock_fd,(char*)buffer,sizeof(distAction_id_t)+sizeof(pingResp),0,(struct sockaddr*)&cnt,len))==-1)  	
  	   {
  	   	exit_on_error("On sending a message in a TCB socket...\n",ret);
  	   }
 	}
 	else
 	{
 	  exit_on_error("On getting a message from a TCB socket...\n",ret);
 	}  
 	 	
	} 	
}

