/*
	(TTCB)
	Distributed Failure Detection Demo Application

	Pedro Martins <pmartins@di.fc.ul.pt>

--[TTCB 1.11]----------------------------------------------------

    (Trusted) Timely Computing Base 
    (C) 1999-2003, Navigators Group 
    		   http://www.navigators.di.fc.ul.pt
    
-----------------------------------------------------------------	
*/

#include "tcb/tcb.h"
#include <signal.h>
#include "defs.h"
#include <sched.h>
#include <netinet/in.h>
#include <linux/inet.h>

char child_stack[1000];
char child_stack_end;

int thrd1;
int sock_fd;

int go_exit=0;

char p[4324];

descriptor_t tcbDesc;
char ip_addr[3000];

int number_of_machines;
char ttcb_id;

void exit_on_error(char * msg,int code)
{
	printf("%s ",msg);

	if (errno==0)
	 printTCBerror(code);
	  else
	   perror(msg);
	
	printf("\n");
	
	kill(thrd1,SIGINT);   
	closeTCBConnection(&tcbDesc);   
	close(sock_fd);
	exit(0);   
}

void sigint_handler(int par)
{
	go_exit=1;
}

void display_fail_info(fail_info_to_app* fi)
{
	int c;
	int mult=1;
	
	printf("********************************************\n");
	printf("Client:\n");
	printf("Message from: %u, Seq: %u Port: %u\n",fi->from,fi->seq,fi->port);
	printf("********************************************\n");
	
	for (c=0;c<number_of_machines;c++)
	 {
	   if (fi->failure_info[c].fail!=2)
	   {
	     if (fi->failure_info[c].fail==1)
	      {
		if (c!=ttcb_id)	
	        	printf("Timing failure on the reception (machine %u)\n",c); 
		else
			printf("Timing failure on the reception (machine %u) LOCAL\n",c); 
	        mult=1;
	      }
	       else
	        {
		if (c!=ttcb_id)			 
	        	printf("Timely reception on machine %u\n",c); 
		else
			printf("Timely reception on machine %u LOCAL\n",c); 
	       	mult=-1;
	        }
	        
	    if (fi->failure_info[c].lateness_avail==1)
	      {
	        printf("lateness degree=%d\n",fi->failure_info[c].lateness_degree*mult); 
	      }
	       else
	        {
	        	printf("Lateness degree not available\n",c); 
	        }    	
	     printf("\n");            
	   }	   
	 }
	 
}

int fn(void* arg)
{
	int ret;	

	fail_info_to_app fi;
	
	while (1)
	{
	  ret = TCBwaitInfo(&tcbDesc,&fi);  	
	  if (ret!=0){
		printf("Error on getting wait info!\n");
	}
   	  display_fail_info(&fi);
	}
	return 0;
}


int main(int argc,char* argv[])
{
	int ret;		
	int one=1;
	TCBtimestamp sndtime,gsndtime,rcv_evt, grcv_evt;
	TCBtimeinterval delay,error;	
 	struct sockaddr_in cnt,serv;
 	int len; 	
 	char buffer[1000];
	unsigned long long cnts;
	pingResp* pt;
	distAction_id_t * distAction;
	fd_set fds;
	struct timeval tv;
	tcb_kernel_info_t krn_info;


	printf("\n----------------------------------------------------------\n");
	printf("Distributed failure detection service - client aplication\n");
	printf("----------------------------------------------------------\n\n");
	
	bzero(&tcbDesc,sizeof(descriptor_t));	
	
	if ((ret = openTCBConnection(&tcbDesc))<OK)
	{
		printTCBerror(ret);
		printf("On opening a connection...\n");
		exit(1);
	}

	// get local TTCB id	
	ttcb_id=getTCBId(&tcbDesc);

	if (ttcb_id < 0){
		closeTCBConnection(&tcbDesc);
		printTCBerror(ret);
		printf("On getting the TTCB id...\n");
		exit(1);
	}
	

	// gather TTCB kernel info
	ret = getTCBKernelInfo(&tcbDesc, &krn_info);

	if (ret < 0){
		closeTCBConnection(&tcbDesc);
		printTCBerror(ret);
		printf("On getting TTCB kernel info...\n");
		exit(1);
	}

	number_of_machines = krn_info.number_of_system_machines;
        	
	//create a thread to read the WaitInfo data  	
	thrd1=clone(&fn,&child_stack_end,CLONE_VM,(void*)0);
	
	signal(SIGINT,sigint_handler);
	
	if ((sock_fd=socket(AF_INET,SOCK_DGRAM,0))<0)
	{
		exit_on_error("Error on creating an UDP socket...\n\n",sock_fd);
	}

	setsockopt(sock_fd,SOL_SOCKET,SO_REUSEADDR|SO_BROADCAST,&one,sizeof(one));
	
 	bzero((char*)&cnt,0,sizeof(cnt));
	
 	cnt.sin_family=AF_INET;
 	cnt.sin_addr.s_addr=htonl(INADDR_ANY);
 	cnt.sin_port=htons(0);
 	
 	if ((ret=bind(sock_fd,(struct sockaddr*)&cnt,sizeof(cnt)))!=0)
 	 {	
 	 	exit_on_error("Error on binding an UDP socket...\n",ret);
 	 }
 	
 	bzero((char*)&serv,0,sizeof(serv));
 	
 	serv.sin_family=AF_INET;
	serv.sin_addr.s_addr=inet_addr(BROADCAST_ADDR);
 	serv.sin_port=htons(16006);
 	
 	cnts=0;
 	
 	while (!go_exit)
 	{
 	 cnts++;
 	 if ((cnts>600000))  
  	 {
  	 ret = getTCBLocalAndGlobalTimestamp(&tcbDesc, &sndtime,&gsndtime); 	 
 	 if (ret < 0){
		exit_on_error("On getting a global timestamp from the TTCB...\n",ret);
	 }

	distAction=(distAction_id_t*)&buffer;
 	ret = startTCBDistWRemoteFD(&tcbDesc,DESTINATION_MASK,SERVER_TCB_PORT,sndtime,gsndtime,SPEC,distAction);
 
 	if ((ret < 0) && (ret!=ALREADY_FAILED)&&(ret!=BURSTY_SENDS)){
		exit_on_error("On starting a distributed FD operation...\n",ret);
		exit(1);
	 }

	 if ((ret!=ALREADY_FAILED)&&(ret!=BURSTY_SENDS))
  	 if ((ret=sendto(sock_fd,(char*)buffer,sizeof(distAction_id_t),0,(struct sockaddr*)&serv,sizeof(serv)))==-1)
  	  {
  	  	exit_on_error("On sending a message to an UDP socket...\n",ret);
  	  }
  	cnts=0;
  	}
	//Non Blocking reading
  	FD_ZERO(&fds);
	FD_SET(sock_fd,&fds);	 		 	
	 	
	tv.tv_sec=0;
	tv.tv_usec=0;
	 	
	ret=select(sock_fd+1,&fds,NULL,NULL,&tv);
	 	
	if (ret)
	{
 	len=sizeof(cnt);
 	if ((ret=recvfrom(sock_fd,(char*)buffer,sizeof(pingResp)+sizeof(distAction_id_t),0,(struct sockaddr*)&cnt,&len))!=-1) 	
 	{
 	 pt=(pingResp*)(buffer+sizeof(distAction_id_t));	 	
 	 printf("------------------------------------\n");	
 	 printf("Reply from %s:\n",inet_ntoa(cnt.sin_addr.s_addr));
 	 printf("------------------------------------\n");
 	 printf("ping resquest time: delay <%llu> us\n",pt->delay/1000);
 	 printf("\n");
 	}	  
 	 else
 	 if (ret<0)  
 	  {
 	     exit_on_error("On getting a message from an UDP socket...\n",ret);
 	  }
 	}  //  !retval  
 	}
 
	kill(thrd1,SIGINT);    	
	closeTCBConnection(&tcbDesc);   
	close(sock_fd);
	exit(0); 	
}

