/*
	(TTCB)
	HelloWorld application

	Pedro Martins <pmartins@di.fc.ul.pt>
	
--[TTCB 1.11]----------------------------------------------------

    (Trusted) Timely Computing Base
    (C) 1999-2003, Navigators Group 
    		   http://www.navigators.di.fc.ul.pt
    	
-----------------------------------------------------------------	
*/	

#include "tcb/tcb.h"
#include <sys/time.h>
#include "tcb/system_defs.h"

descriptor_t tcbDesc;

int main()
{
	TCBtimestamp start_evt;
	TCBtimestamp global_ts;
	int tcb_id;
	int ret;
	tcb_kernel_info_t krn_info;
	
	bzero(&tcbDesc,sizeof(descriptor_t));		        		
	
	// open a conection to the TCB
	if ( (ret=openTCBConnection(&tcbDesc))<OK)
	{
		printTCBerror(ret);
		printf("On opening a connection...\n");
		exit(1);
	}
	
	// get local TCB id	
	tcb_id=getTCBId(&tcbDesc);

	if (tcb_id < 0){
		closeTCBConnection(&tcbDesc);
		printTCBerror(ret);
		printf("On getting the TCB id...\n");
		exit(1);
	}
	
	printf("Conection to TCB local node (%u) established\n",tcb_id);

	// gather and display TCB kernel info
	ret = getTCBKernelInfo(&tcbDesc, &krn_info);

	if (ret < 0){
		closeTCBConnection(&tcbDesc);
		printTCBerror(ret);
		printf("On getting TCB kernel info...\n");
		exit(1);
	}

	printf("TTCB version: %d\n",krn_info.version);
	printf("TTCB subversion: %d\n",krn_info.sub_version);
	printf("TTCB extentions : %d\n",krn_info.ttcb_extentions);

	// a local timestamp
	ret = getTCBTimestamp(&tcbDesc, &start_evt);
	
	if (ret < 0){
		closeTCBConnection(&tcbDesc);
		printTCBerror(ret);
		printf("On getting a local timestamp...\n");
		exit(1);
	}

	printf("A local TCB timestamp : %lld\n",start_evt);	

#ifdef DISTRIBUTED			
	// a global timestamp		
	ret = getTCBGlobalTimestamp(&tcbDesc, &global_ts);	
	
	if (ret < 0){
		closeTCBConnection(&tcbDesc);		
		printTCBerror(ret);
		printf("On getting a global timestamp...\n");
		exit(1);
	}
	printf("A global TCB timestamp : %lld\n",global_ts);	
#endif
	ret = closeTCBConnection(&tcbDesc);

	if (ret < 0){
		closeTCBConnection(&tcbDesc);		
		printTCBerror(ret);
		printf("On closing the connection...\n");
		exit(1);
	}
	
	return 0;
}

