/*
	(TTCB)
	Local Timely Failure Detection Demo Application	

	Pedro Martins <pmartins@di.fc.ul.pt>

--[TTCB 1.11]----------------------------------------------------

    (Trusted) Timely Computing Base 
    (C) 1999-2003, Navigators Group 
    		   http://www.navigators.di.fc.ul.pt
    
------------------------------------------------------------------	
*/

	
#include "tcb/tcb.h"
#include <sys/time.h>
#include <sys/signal.h>


descriptor_t tcbDesc;

int go_exit=0;

void sig_int(int s)
{
	go_exit=1;	
}

int main()
{
	int res,desc_lfd,ret;
	TCBtimestamp start_evt,end_evt;
	TCBtimeinterval spec,deadline,proc_delay;
	Func_Id funcId;
	char faulty;
	unsigned long long estimated=0;
	
	signal(SIGINT,sig_int);	

	bzero(&tcbDesc,sizeof(descriptor_t));		        		
	
	// open a conection to TCB
	if ( (ret=openTCBConnection(&tcbDesc))<OK)
	{
		printTCBerror(ret);
		printf("On opening a connection...\n");
		exit(1);
	}

	
	printf("During the execution press <ctrl+c> to exit\n");
	printf("Press <enter> to continue\n");
	getchar();
	
	spec=estimated;	
	deadline=200000;       // 200 microseconds
				
	strcpy(funcId.module,"app_moduleL");
	strcpy(funcId.funcname,"timelyExecL");
	strcpy(funcId.wcetname,"timelyExec_wcetL");
	
	while (!go_exit)
	{
	ret = getTCBTimestamp(&tcbDesc, &start_evt);
	
	if (ret < 0){
		closeTCBConnection(&tcbDesc);
		printTCBerror(ret);
		printf("On getting a local timestamp...\n");
		exit(1);
	}

	desc_lfd = startTCBLocalFD(&tcbDesc,start_evt,spec,deadline,funcId);

	if ((desc_lfd<0)&&(desc_lfd!=ALREADY_FAILED))
	{	
		closeTCBConnection(&tcbDesc);
		printTCBerror(desc_lfd);
		printf("On starting a local failure detection operation...\n");
		exit(1);
	}	
	
	if (desc_lfd==ALREADY_FAILED)
	{
		estimated+=10000;
		spec=estimated;
	}
	 else
	  {
	  	ret = endTCBLocalFD(&tcbDesc,desc_lfd,&end_evt,&proc_delay,&faulty);
	  	if (ret<0)
		{	
			closeTCBConnection(&tcbDesc);
			printTCBerror(ret);
			printf("On finishing a local failure detection operation...\n");
			exit(1);	
		}
		
		if (faulty)
		{
			printf("<FAILURE> SPEC: %llu us DURATION: %llu us\n",spec/1000,proc_delay/1000);
			estimated=estimated+=10000;
			spec=estimated;						
		}
		 else
		  {
			printf("SPEC: %llu us DURATION: %llu us\n",spec/1000,proc_delay/1000);
		  }
	}
	sleep(1);
	}

	closeTCBConnection(&tcbDesc);
	
	return 0;
}
