/*
	(TTCB)
	Local Measurement demo application

	Pedro Martins <pmartins@di.fc.ul.pt>
	
--[TTCB 1.11]----------------------------------------------------

    (Trusted) Timely Computing Base 
    (C) 1999-2003, Navigators Group 
    		   http://www.navigators.di.fc.ul.pt
    
-----------------------------------------------------------------	
*/	

#include "tcb/tcb.h"
#include <sys/time.h>
#include <sys/signal.h>


descriptor_t tcbDesc;

void sig_int(int s)
{	
	int ret;

	ret = closeTCBConnection(&tcbDesc);

	if (ret < 0){
		closeTCBConnection(&tcbDesc);		
		printTCBerror(ret);
		printf("On closing the connection...\n");
		exit(1);
	}
			
	exit(0);
}

int main()
{
	int res,desc_ms,ret;
	TCBtimestamp start_evt,end_evt;
	TCBtimeinterval proc_delay;
	
	signal(SIGINT,sig_int);	

	bzero(&tcbDesc,sizeof(descriptor_t));		        		
	
	// open a conection to TCB
	if ( (ret=openTCBConnection(&tcbDesc))<OK)
	{
		printTCBerror(ret);
		printf("On opening a connection...\n");
		exit(1);
	}
		
	
	printf("During the execution press <ctrl+c> to exit\n");
	printf("Press <enter> to continue\n");
	getchar();
	
	while (1)
	{
	
	 ret = getTCBTimestamp(&tcbDesc, &start_evt);
	
	if (ret < 0){
		closeTCBConnection(&tcbDesc);
		printTCBerror(ret);
		printf("On getting a local timestamp...\n");
		exit(1);
	}
			
	desc_ms = startTCBMeasurement(&tcbDesc,start_evt);
	
	if ((desc_ms<0))
	{	
		closeTCBConnection(&tcbDesc);
		printTCBerror(ret);
		printf("On starting a local measurement...\n");
		exit(1);
	}	
	
	/* the measured computation should be here
		...
		...	
	*/

	ret = endTCBMeasurement(&tcbDesc,desc_ms,&proc_delay,&end_evt);
	if (ret<0)
	{	
		closeTCBConnection(&tcbDesc);
		printTCBerror(ret);
		printf("On finishing a local measurement...\n");
		exit(1);
	}	
	printf("-----------\n");	
	printf("Measurement:\n");
	printf("-----------\n");	
	printf("Start Event : %lld\n",start_evt);	
	printf("End Event   : %lld\n",end_evt);		
	printf("Measurement : %lld\n",proc_delay);		
	
	sleep(1);
	}
	
	return 0;
}

