/*
	(TTCB)
	Timestamping application

	Pedro Martins <pmartins@di.fc.ul.pt>
	
--[TTCB 1.11]----------------------------------------------------

    (Trusted) Timely Computing Base
    (C) 1999-2003, Navigators Group 
    		   http://www.navigators.di.fc.ul.pt
    
------------------------------------------------------------------	
*/	

#include "tcb/tcb.h"
#include <sys/time.h>
#include "tcb/system_defs.h"

descriptor_t tcbDesc;

int main()
{
	TCBtimestamp local_ts;
	TCBtimestamp global_ts;	
	int tcb_id;
	int ret;

	bzero(&tcbDesc,sizeof(descriptor_t));		        			
		
	// open a conection to TTCB
	if ( (ret=openTCBConnection(&tcbDesc))<OK)
	{
		printTCBerror(ret);
		printf("On opening a connection...\n");
		exit(1);	
	}
	
	// get local TTCB id	
	tcb_id=getTCBId(&tcbDesc);

	if (tcb_id < 0){
		closeTCBConnection(&tcbDesc);
		printTCBerror(ret);
		printf("On getting the TTCB id...\n");
		exit(1);
	}
	
	printf("Connection to TTCB local node (%u) established\n",tcb_id);
		
	// local timestamp		
	ret = getTCBTimestamp(&tcbDesc, &local_ts);
	
	if (ret < 0){
		closeTCBConnection(&tcbDesc);
		printTCBerror(ret);
		printf("On getting a local timestamp...\n");
		exit(1);
	}
	printf("A local TTCB timestamp : %lld\n",local_ts);	
#ifdef DISTRIBUTED
	// gloal timestamp
	ret = getTCBGlobalTimestamp(&tcbDesc, &global_ts);

	if (ret < 0){
		closeTCBConnection(&tcbDesc);		
		printTCBerror(ret);
		printf("On getting a global timestamp...\n");
		exit(1);
		}
	printf("A global TTCB timestamp : %lld\n",global_ts);	
#endif		
	
	closeTCBConnection(&tcbDesc);
	
	return 0;
}
