
#ifndef _RTnetdevice
#define _RTnetdevice 
 
#include <linux/netdevice.h>
#include "RTskbuff.h"
 
 
//MUDAR ISTO!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 
struct RTnet_device_stats
{
	unsigned long	rx_packets;		/* total packets received	*/
	unsigned long	tx_packets;		/* total packets transmitted	*/
	unsigned long	rx_bytes;		/* total bytes received 	*/
	unsigned long	tx_bytes;		/* total bytes transmitted	*/
	unsigned long	rx_errors;		/* bad packets received		*/
	unsigned long	tx_errors;		/* packet transmit problems	*/
	unsigned long	rx_dropped;		/* no space in linux buffers	*/
	unsigned long	tx_dropped;		/* no space available in linux	*/
	unsigned long	multicast;		/* multicast packets received	*/
	unsigned long	collisions;

	/* detailed rx_errors: */
	unsigned long	rx_length_errors;
	unsigned long	rx_over_errors;		/* receiver ring buff overflow	*/
	unsigned long	rx_crc_errors;		/* recved pkt with crc error	*/
	unsigned long	rx_frame_errors;	/* recv'd frame alignment error */
	unsigned long	rx_fifo_errors;		/* recv'r fifo overrun		*/
	unsigned long	rx_missed_errors;	/* receiver missed packet	*/

	/* detailed tx_errors */
	unsigned long	tx_aborted_errors;
	unsigned long	tx_carrier_errors;
	unsigned long	tx_fifo_errors;
	unsigned long	tx_heartbeat_errors;
	unsigned long	tx_window_errors;
	
	unsigned long	rx_compressed;
	unsigned long	tx_compressed;	
};

struct RTnet_device
{

	char			name[IFNAMSIZ];

	/*
	 *	I/O specific fields
	 *	FIXME: Merge these and struct ifmap into one
	 */
#if 0
	unsigned long		rmem_end;	/* shmem "recv" end	*/
	unsigned long		rmem_start;	/* shmem "recv" start	*/
	unsigned long		mem_end;	/* shared mem end	*/
	unsigned long		mem_start;	/* shared mem start	*/
#endif
	unsigned long		base_addr;	/* device I/O address	*/
	unsigned int		irq;		/* device IRQ number	*/

	/*
	 *	Some hardware also needs these fields, but they are not
	 *	part of the usual set specified in Space.c.
	 */
	atomic_t		is_receiving;    /* flag that indicates if incomming
						   packets are to be processed
						*/
	unsigned char		if_port;	/* Selectable AUI, TP,..*/
	unsigned char		dma;		/* DMA channel		*/

	unsigned long		state;

	struct RTnet_device	*next;
	
	/* The device initialization function. Called only once. */
	int			(*init)(struct RTnet_device *dev);

	/* ------- Fields preinitialized in Space.c finish here ------- */

#if 0
	struct RTnet_device	*next_sched;
#endif

	/* Interface index. Unique device identifier	*/
	int			ifindex;
	int			iflink;


	struct RTnet_device_stats* (*get_stats)(struct RTnet_device *dev);
#if 0
	struct iw_statistics*	(*get_wireless_stats)(struct net_device *dev);
#endif
	/*
	 * This marks the end of the "visible" part of the structure. All
	 * fields hereafter are internal to the system, and may change at
	 * will (read: may be cleaned up at will).
	 */

	/* These may be needed for future network-power-down code. */
	unsigned long		trans_start;	/* Time (in jiffies) of last Tx	*/
	unsigned long		last_rx;	/* Time of last Rx	*/

	unsigned short		flags;	/* interface flags (a la BSD)	*/
	unsigned short		gflags;
	unsigned		mtu;	/* interface MTU value		*/
	unsigned short		type;	/* interface hardware type	*/
	unsigned short		hard_header_len;	/* hardware hdr length	*/
	void			*priv;	/* pointer to private data	*/

#if 0
	struct net_device	*master; /* Pointer to master device of a group,
					  * which this device is member of.
					  */
#endif
	/* Interface address info. */
	unsigned char		broadcast[MAX_ADDR_LEN];	/* hw bcast add	*/
	unsigned char		pad;		/* make dev_addr aligned to 8 bytes */
	unsigned char		dev_addr[MAX_ADDR_LEN];	/* hw address	*/
	unsigned char		addr_len;	/* hardware address length	*/
	struct dev_mc_list	*mc_list;	/* Multicast mac addresses	*/
	int			mc_count;	/* Number of installed mcasts	*/
#if 0
	int			promiscuity;
	int			allmulti;

	int			watchdog_timeo;
	struct timer_list	watchdog_timer;

	/* Protocol specific pointers */
	
	void 			*atalk_ptr;	/* AppleTalk link 	*/
	void			*ip_ptr;	/* IPv4 specific data	*/  
	void                    *dn_ptr;        /* DECnet specific data */
	void                    *ip6_ptr;       /* IPv6 specific data */
	void			*ec_ptr;	/* Econet specific data	*/

	struct Qdisc		*qdisc;
	struct Qdisc		*qdisc_sleeping;
	struct Qdisc		*qdisc_list;
	struct Qdisc		*qdisc_ingress;
#endif
	unsigned long		tx_queue_len;	/* Max frames per queue allowed */

	/* hard_start_xmit synchronizer */
	spinlock_t		xmit_lock;
	/* cpu id of processor entered to hard_start_xmit or -1,
	   if nobody entered there.
	 */
	int			xmit_lock_owner;

#if 0
	/* device queue lock */
	spinlock_t		queue_lock;

	/* Number of references to this device */
	atomic_t		refcnt;
	/* The flag marking that device is unregistered, but held by an user */
	int			deadbeaf;
#endif
	/* Net device features */
	int			features;
#define NETIF_F_SG		1	/* Scatter/gather IO. */
#define NETIF_F_IP_CSUM		2	/* Can checksum only TCP/UDP over IPv4. */
#define NETIF_F_NO_CSUM		4	/* Does not require checksum. F.e. loopack. */
#define NETIF_F_HW_CSUM		8	/* Can checksum all the packets. */
#define NETIF_F_DYNALLOC	16	/* Self-dectructable device. */
#define NETIF_F_HIGHDMA		32	/* Can DMA to high memory. */
#define NETIF_F_FRAGLIST	1	/* Scatter/gather IO. */

#if 0
	/* Called after device is detached from network. */
	void			(*uninit)(struct net_device *dev);
	/* Called after last user reference disappears. */
	void			(*destructor)(struct net_device *dev);
#endif
	/* Pointers to interface service routines.	*/
	int			(*open)(struct RTnet_device *dev);
	int			(*stop)(struct RTnet_device *dev);
	int			(*hard_start_xmit) (struct RTsk_buff *skb,
						    struct RTnet_device *dev);
	int			(*hard_header) (struct RTsk_buff *skb,
						struct RTnet_device *dev,
						unsigned short type,
						void *daddr,
						void *saddr,
						unsigned len);
			
#if 0			
	int			(*rebuild_header)(struct RTsk_buff *skb);
#define HAVE_MULTICAST			 
	void			(*set_multicast_list)(struct RTnet_device *dev);
#define HAVE_SET_MAC_ADDR  		 
	int			(*set_mac_address)(struct net_device *dev,
						   void *addr);
#define HAVE_PRIVATE_IOCTL
	int			(*do_ioctl)(struct net_device *dev,
					    struct ifreq *ifr, int cmd);
#define HAVE_SET_CONFIG
	int			(*set_config)(struct net_device *dev,
					      struct ifmap *map);
#define HAVE_HEADER_CACHE
	int			(*hard_header_cache)(struct neighbour *neigh,
						     struct hh_cache *hh);
	void			(*header_cache_update)(struct hh_cache *hh,
						       struct net_device *dev,
						       unsigned char *  haddr);
#define HAVE_CHANGE_MTU
	int			(*change_mtu)(struct net_device *dev, int new_mtu);

#define HAVE_TX_TIMEOUT
	void			(*tx_timeout) (struct net_device *dev);

	int			(*hard_header_parse)(struct sk_buff *skb,
						     unsigned char *haddr);
	int			(*neigh_setup)(struct net_device *dev, struct neigh_parms *);
	int			(*accept_fastpath)(struct net_device *, struct dst_entry*);

	/* open/release and usage marking */
	struct module *owner;

	/* bridge stuff */
	struct net_bridge_port	*br_port;

#ifdef CONFIG_NET_FASTROUTE
#define NETDEV_FASTROUTE_HMASK 0xF
	/* Semi-private data. Keep it at the end of device struct. */
	rwlock_t		fastpath_lock;
	struct dst_entry	*fastpath[NETDEV_FASTROUTE_HMASK+1];
#endif
#ifdef CONFIG_NET_DIVERT
	/* this will get initialized at each interface type init routine */
	struct divert_blk	*divert;
#endif /* CONFIG_NET_DIVERT */

#endif

};

/*
	netif_xxx queues routines that are not implemented yet.
	used to provide an uniform API to device drivers
*/

static inline void RTnetif_schedule(struct RTnet_device *dev)
{
	//if (!test_bit(__LINK_STATE_XOFF, &dev->state))
	//	__netif_schedule(dev);
}

static inline void RTnetif_start_queue(struct RTnet_device *dev)
{
	//clear_bit(__LINK_STATE_XOFF, &dev->state);
}

static inline void RTnetif_wake_queue(struct RTnet_device *dev)
{
	//if (test_and_clear_bit(__LINK_STATE_XOFF, &dev->state))
	//	__netif_schedule(dev);
}

static inline void RTnetif_stop_queue(struct RTnet_device *dev)
{
	//set_bit(__LINK_STATE_XOFF, &dev->state);
}

static inline int RTnetif_queue_stopped(struct RTnet_device *dev)
{
	//return test_bit(__LINK_STATE_XOFF, &dev->state);
	return 0;
}

static inline int RTnetif_running(struct RTnet_device *dev)
{
	//return test_bit(__LINK_STATE_START, &dev->state);
	return 1;
}

/* Hot-plugging. */
static inline int RTnetif_device_present(struct RTnet_device *dev)
{
	//return test_bit(__LINK_STATE_PRESENT, &dev->state);
	return 1;
}

#endif



