/*
 *	mem_mng.c
 *
 *	Pedro Martins	<pmartins@di.fc.ul.pt>
 *	
 *	Memory management support for RTIP
 *
 * 	Last Review Date: June 2003
*/

#include "rtip/mem_mng.h"
#include <rtai.h>

/*
 * Real-Time/Interrupt mode: Not Allowed
*/
int init_obj_list(object_list* olist, int n_items, int size_item){
	
	int a;
	pt_obj_list list;	

	spin_lock_init(olist->slock);

	olist->sizeof_item = size_item;
	olist->n_items = n_items;

	list = kmalloc (size_item * n_items, GFP_ATOMIC);

	olist->list = list;

	if (list == NULL)	
		return -ENOMEM;
	
	//set links	
	list->prev_free = (list+n_items-1);
	list->id = 0;
	list->next_free = (list+1);
	list->available = 1;
	list->object = NULL;
	list->magic = RTSKBUFF_MAGIC_NUMBER;
	if ((list->object=kmalloc(size_item,GFP_ATOMIC))==NULL){
		kfree(list);
		return -ENOMEM;
	}
	for (a=1; a<n_items; a++){
		(list+a)->id = a;
		(list+a)->object = NULL;
		(list+a)->magic = RTSKBUFF_MAGIC_NUMBER;
		(list+a)->next_free = (list+a+1);
		(list+a)->prev_free = (list+a-1);
		(list+a)->available = 1;
		if (((list+a)->object=kmalloc(size_item,GFP_ATOMIC))==NULL){
		for (a--;a>=0;a--){
			kfree(list->object);
		}
		kfree(list);
		return -ENOMEM;
	}
	}
	(list+a-1)->next_free = list;

	olist->free_list = list;

	return 0;
}

/*
 * Real-Time/Interrupt mode: Not Allowed
 * Make sure that no racings occur
*/
void free_obj_list(object_list* olist){	
	
	int a;

	for (a=0; a<olist->n_items; a++){
		kfree(((olist->list)+a)->object);
	}

	kfree (olist->list); 
}

void free_obj_list_item(object_list* olist, pt_obj_list rec){

	unsigned int irqflags;

	if (rec->magic != RTSKBUFF_MAGIC_NUMBER)
	{
		rt_printk(KERN_ERR "magic number doesn't match in 'free_obj_list_item'\n");
		return;
	}
	
	if (rec->available) 
		return ;
	
	rec->available = 1;

	irqflags = rt_spin_lock_irqsave(olist->slock);

	//update free list pointers	
	if (olist->free_list == NULL){ 
		rec->next_free = rec; 
		rec->prev_free = rec;
		olist->free_list = rec;
	} else {	
		rec->next_free = olist->free_list->next_free;
		rec->prev_free = olist->free_list;
		(olist->free_list->next_free)->prev_free = rec;
		olist->free_list->next_free = rec;		
	}

	rt_spin_unlock_irqrestore(irqflags,olist->slock);
}

pt_obj_list get_obj_list_item(object_list* olist){

	unsigned int irqflags;

	pt_obj_list ret;	

	irqflags = rt_spin_lock_irqsave(olist->slock);

	if (!olist->free_list){
	        rt_spin_unlock_irqrestore(irqflags,olist->slock);
		return NULL;
	}
	
	//returns the first free entry to user
	ret = olist->free_list;

	if (olist->free_list->next_free == olist->free_list){
	        //ret = olist->free_list;
		olist->free_list = NULL;
		goto out;
	}

	//update free list pointers	
	(olist->free_list->next_free)->prev_free = olist->free_list->prev_free;
	(olist->free_list->prev_free)->next_free = olist->free_list->next_free;

	olist->free_list = ret -> next_free;
out:
	ret->available = 0;
	rt_spin_unlock_irqrestore(irqflags,olist->slock);
	return ret;
	
	
}

void	debug_display_obj_list(object_list* olist){		
	int a;
	unsigned int irqflags;
	pt_obj_list init;	
	pt_obj_list act;
	pt_obj_list list;

	irqflags = rt_spin_lock_irqsave(olist->slock);	

	init = olist->free_list;	
	act = olist->free_list;
	list = olist->list;

	
	printk(KERN_DEBUG "RTsk_buff cache: <%d> buffers\n\n",olist->n_items);	
	printk(KERN_DEBUG "Allocation Info:\n");	
	for (a=0;a<olist->n_items;a++){
		printk(KERN_DEBUG " buffer %d: %s\n",a,(list+a)->available? "free" : "alloc");
	}

	printk(KERN_DEBUG "\nFree List ptr's:");

	do {
		if (act != NULL) {
			printk(KERN_DEBUG "%d - ",act->id);
			act = act->next_free;
		}
	}
	while (act!=init);
	printk(KERN_DEBUG "\n");

	rt_spin_unlock_irqrestore(irqflags,olist->slock);		
}

