/*
 *	RTdev.c
 *
 *	RT-INET 1.0
 *
 *	Pedro Martins <pmartins@di.fc.ul.pt>
 *	Device initialization routines
 *
 * 	Last Review Date: August 2003
*/



//TODO: rx_drooped nas estatisticas

#include "rtip/RTdev.h"
#include <linux/module.h>

#include <rtai_sched.h>
#include <rtai.h>

#include "rtip/mem_mng.h"

//pending rx packtes list
object_list	backlog;
	
typedef struct backlog_rec {
	RTsk_buff* buff;
	struct backlog_rec* next;
	struct backlog_rec* prev;
	pt_obj_list olist_rec;	
}bckl;

//packets are inverted in the backlog (concerning their receipts)
struct backlog_rec*  pending_packets = NULL;
struct backlog_rec*  oldest_pending_packet = NULL;
spinlock_t	backlog_spinl;

//list off all real-time network devices
struct RTnet_device* RTnet_device_list=NULL;


//loopback device
struct RTnet_device loopback = 
	{"RTloopback", 0, 0};


/* 
 *************************
 * temporary API for TCB *
 *************************
 */

int RTsendMessage(char *buff, int size){			
	struct RTsk_buff*	skbuff,*cloned_skbuff = NULL;
	char *pt;

	skbuff = RTdev_alloc_skb();

	if (skbuff == NULL){
		rt_printk("No available RTskbuff for send...\n");
		return -1;
	}
		
	pt = RTskb_put(skbuff,size);
	
	if (pt == NULL){
		return 1;
	}	
	memcpy(pt,buff,size);

	if (RTnet_device_list!=NULL){
		cloned_skbuff = RTskb_copy(skbuff);
		if (cloned_skbuff==NULL){
			rt_printk("No available RTskbuff(copy) for send...\n");
			return -1;
		}
	}

	//build frame header
	loopback.hard_header(skbuff,&loopback,0,NULL,NULL,size);

	//send on loopback
	loopback.hard_start_xmit(skbuff,&loopback);

	if (RTnet_device_list!=NULL){
		//send in broadcast	
		RTnet_device_list->hard_header(cloned_skbuff,RTnet_device_list,ETH_P_802_3,RTnet_device_list->broadcast,NULL,size);
		RTnet_device_list->hard_start_xmit(cloned_skbuff,RTnet_device_list);
	}
	
	return 0;	
}

RTsk_buff* RTreadMessage(void){
	struct backlog_rec *oldest;
	RTsk_buff* ret;
	unsigned int irqflags;
	
	irqflags = rt_spin_lock_irqsave(&backlog_spinl);

        oldest = oldest_pending_packet; 		 		
        if (oldest==NULL){
        	rt_spin_unlock_irqrestore(irqflags,&backlog_spinl);
        	return NULL;
        }
        oldest_pending_packet = oldest_pending_packet->prev; 		 			
        
        if (oldest_pending_packet == NULL)
        	pending_packets = NULL;
        
	ret = oldest->buff;	
	free_obj_list_item(&backlog,oldest->olist_rec);		
	rt_spin_unlock_irqrestore(irqflags,&backlog_spinl);
	return ret;
}

void RTreleaseMessage(RTsk_buff* buff){	
	RTkfree_skb(buff);
}

int RTgetMTU(void){
	int lo_mtu = loopback.mtu;

	//at now it just supports one device
	if (RTnet_device_list!=NULL){
		if (RTnet_device_list->mtu < lo_mtu)
	  	  return RTnet_device_list->mtu;
	}
	return lo_mtu;
}

u64 RTgetMacAddr(void){

	if (RTnet_device_list!=NULL){
		return (
	          ((u64)RTnet_device_list->dev_addr[0] <<40)+
	          ((u64)RTnet_device_list->dev_addr[1] <<32)+
	          ((u64)RTnet_device_list->dev_addr[2] <<24)+
	          ((u64)RTnet_device_list->dev_addr[3] <<16)+
	          ((u64)RTnet_device_list->dev_addr[4] <<8)+
	          ((u64)RTnet_device_list->dev_addr[5]) 
	         );
	}
	else
	  return 0;  //loopback device MAC addr
}

void RTstartReceive (void){
	
	//at now it just supports one driver
	if (RTnet_device_list!=NULL)
		atomic_set(&(RTnet_device_list-> is_receiving),1); 
	
	atomic_set(&(loopback.is_receiving),1); 
}

void RTstopReceive (void){
	
	//at now it just supports one driver
	if (RTnet_device_list!=NULL)
		atomic_set(&(RTnet_device_list-> is_receiving),0); 
	
	atomic_set(&(loopback.is_receiving),0); 
}

/* ****************************************** */


void RTnet_dev_get_info(char* buff, struct RTnet_device** next_dev, struct RTnet_device* ndev){
	char b[200];
	struct RTnet_device* dev=NULL;
	struct RTnet_device_stats *stats=NULL;

	if (ndev==NULL){
		dev = &loopback;
		stats = loopback.get_stats(&loopback);
		*next_dev=RTnet_device_list;
	}
	else {
		dev = *next_dev;
		stats = dev->get_stats(dev);
		*next_dev=NULL;
	}

	strcpy(buff,"");

	sprintf(b,"%s:\n",dev->name);
	strcat(buff,b);
	sprintf(b,"     MTU: %d \n",dev->mtu);
	strcat(buff,b);
	sprintf(b,"     RX packets:%lu errors:%lu dropped:%lu fifo_overruns:%lu frame:%lu\n",stats->rx_packets,stats->rx_errors,stats->rx_dropped,stats->rx_fifo_errors,stats->rx_frame_errors);
	strcat(buff,b);
	sprintf(b,"        buffer_overruns:%lu crc_errors:%lu\n",stats->rx_over_errors,stats->rx_crc_errors);
	strcat(buff,b);																
	sprintf(b,"     TX packets:%lu errors:%lu dropped:%lu fifo_overruns:%lu carrier:%lu\n",stats->tx_packets,stats->tx_errors,stats->tx_dropped,stats->tx_fifo_errors,stats->tx_carrier_errors);
	strcat(buff,b);
	sprintf(b,"     collisions:%lu\n",stats->collisions);
	strcat(buff,b);
	sprintf(b,"     RX bytes:%lu  TX bytes:%lu\n",stats->rx_bytes,stats->tx_bytes);
	strcat(buff,b);
	strcat(buff,"\n");

        strcat(buff,"\0");	
       	
}


int RTnetif_rx(struct RTsk_buff *skb){

	pt_obj_list pt;
	struct backlog_rec *obj;
	unsigned int irqflags;


	if (atomic_read(&(skb->dev->is_receiving))==0){
		RTkfree_skb(skb);
	}
	else {

	//get a timestmap for receipt
	skb->timestamp = rt_get_time_ns();

	irqflags = rt_spin_lock_irqsave(&backlog_spinl);

	pt = get_obj_list_item(&backlog);	
		
	if (pt == NULL){
		rt_printk("RTIP: rx packet lost due to an overflow in the backlog rx queue...\n");
		rt_spin_unlock_irqrestore(irqflags,&backlog_spinl);	
//actualizar rx_dropped !!!!
		return -1;		
	}

	obj = (struct backlog_rec*)pt->object;

	obj->buff = skb;
	obj->olist_rec = pt;
	obj->next = pending_packets;
	obj->prev = NULL;

	if (pending_packets!=NULL)
		pending_packets->prev=obj;
		 else
	 	  oldest_pending_packet = obj;

	pending_packets = obj;

	rt_spin_unlock_irqrestore(irqflags,&backlog_spinl);
	}

	return 0;
}

int RTregister_netdevice(struct RTnet_device *dev){

	//to be changed
	if (RTnet_device_list == NULL){		
		dev->open(dev);
		RTnet_device_list=dev;
	}		
	return 0;
}

int RTunregister_netdevice(struct RTnet_device *dev){
	
	//to be changed
	if (RTnet_device_list != NULL){
		dev->stop(dev);
	}
	RTnet_device_list=NULL;
	return 0;
}

int RTdev_alloc_name(struct RTnet_device *dev, char *name){

	//by now, it just supports one device			
	strcpy(dev->name,"RTeth0");
	return 6;
}

int rtnet_dev_init(int n_buffs){

	if (init_obj_list(&backlog,n_buffs,sizeof(struct backlog_rec))){
		return -ENOMEM;
	}
	spin_lock_init(&backlog_spinl);
	return 0;
}

void rtnet_dev_close(void){

	free_obj_list(&backlog);
}
