
/*
 *	RTskbuff.c
 *	based on <net/core/skbuff.c>	
 *
 *	RT-INET 1.0
 *
 *	Pedro Martins <pmartins@di.fc.ul.pt>
 * 	
 *	Real-Time Socket Buffers	
 *
 * 	Last Review Date: June 2003
 */

#include "rtip/RTskbuff.h"

object_list RTskbuffs; 

void RTskb_copy_and_csum_dev(const struct RTsk_buff *skb, u8 *to)
{
//	unsigned int csum;
	long csstart;

//	if (skb->ip_summed == CHECKSUM_HW)
//		csstart = skb->h.raw - skb->data;
//	else
//		csstart = skb->len - skb->data_len;
	csstart = skb->len;

//	if (csstart > skb->len - skb->data_len)
//		BUG();

	memcpy(to, skb->data, csstart);

/*
	csum = 0;
	if (csstart != skb->len)
		csum = RTskb_copy_and_csum_bits(skb, csstart, to+csstart,
				skb->len-csstart, 0);
*/		

//	if (skb->ip_summed == CHECKSUM_HW) {
//		long csstuff = csstart + skb->csum;

//	*((unsigned short *)(to + csstuff)) = csum_fold(csum);
//	}
}


//NOTE: by now, it releases all the sk_buff struct, not only the data.
static void RTskb_release_data(struct RTsk_buff *skb)
{
	if (!skb->cloned ||
	    atomic_dec_and_test(RTskb_dataref(skb))) {
		free_obj_list_item(&RTskbuffs,skb->obj_list);
	}
}


/*
 *	Free an RTskbuff by memory without cleaning the state. 
 */
void RTkfree_skbmem(struct RTsk_buff *skb)
{
	RTskb_release_data(skb);
//	RTskb_head_to_pool(skb);
}

/**
 *	__RTkfree_skb - private function 
 *	@skb: buffer
 *
 *	Free an RTsk_buff. Release anything attached to the buffer. 
 *	Clean the state. This is an internal helper function. Users should
 *	always call kfree_skb
 */

void __RTkfree_skb(struct RTsk_buff *skb)
{
/*
	if (skb->list) {
	 	rt_printk("Warning: RTkfree_skb passed an skb still on a list\n");
	}
*/

//	dst_release(skb->dst);
//	skb_headerinit(skb, NULL, 0);  /* clean state */
	RTkfree_skbmem(skb);
}


/**
 *	RTskb_clone	-	duplicate an RTsk_buff
 *	@skb: buffer to clone
 *
 *	Duplicate an &RTsk_buff. The new one is not owned by a socket. Both
 *	copies share the same packet data but not structure. The new
 *	buffer has a reference count of 1. If the allocation fails the 
 *	function returns %NULL otherwise the new buffer is returned.
 *	
 */

struct RTsk_buff *RTskb_clone(struct RTsk_buff *skb)
{
	struct RTsk_buff *n;
	pt_obj_list pt;	

	n = NULL;
	pt = get_obj_list_item(&RTskbuffs);
	if (pt!=NULL)
		n = (RTsk_buff*)pt->object;
	if (n == NULL) {
			return NULL;
	}

#define C(x) n->x = skb->x

//	n->next = n->prev = NULL;
//	n->list = NULL;
//	n->sk = NULL;
	C(timestamp);
	C(dev);
	C(h);
	C(nh);
	C(mac);
	C(len);
	n->cloned = 1;
	C(pkt_type);
	C(ip_summed);
	C(priority);
	atomic_set(&n->users, 1);
	C(protocol);
	C(truesize);
	C(head);
	C(data);
	C(tail);
	C(end);
	C(obj_list);

	atomic_inc(RTskb_dataref(skb));
	skb->cloned = 1;

	n->obj_list = pt;

	return n;
}
 
/**
 *	RTalloc_skb	-	allocate a RT network buffer
 *
 *	Allocate a new &RTsk_buff. The returned buffer has no headroom and a
 *	tail room of SKBUFF_PKT_SIZE bytes. The object has a reference count of 
 *	one.
 *	The return is the buffer. On a failure the return is %NULL.
 *
 */ 
 
struct RTsk_buff *RTalloc_skb(void){

	RTsk_buff *skb;
	pt_obj_list pt;	

	skb = NULL;
	pt = get_obj_list_item(&RTskbuffs);
	if (pt!=NULL)
		skb = (RTsk_buff*)pt->object;
	if (skb == NULL) {
			return NULL;
	}
	skb->truesize = sizeof(struct RTsk_buff);

	/* Load the data pointers. */
	skb->data = skb->bdata;
	skb->head = skb->bdata;
	skb->tail =  skb->bdata;
	skb->end = skb->bdata + SKBUFF_PKT_SIZE;

	/* Set up other state */
	skb->len = 0;
	skb->obj_list = pt;

	skb->timestamp=0;		/* No idea about time */
	skb->pkt_type = PACKET_HOST;	/* Default type */
	skb->ip_summed = 0;
	skb->priority = 0;
	skb->cloned = 0;
	atomic_set(&skb->users, 1);
	atomic_set(RTskb_dataref(skb), 1);

	return skb;
};


 /**
 *	RTskb_copy	-	create private copy of an RTsk_buff
 *	@skb: buffer to copy
 *
 *	Make a copy of both an &RTsk_buff and its data. This is used when the
 *	caller wishes to modify the data and needs a private copy of the 
 *	data to alter. Returns %NULL on failure or the pointer to the buffer
 *	on success. The returned buffer has a reference count of 1.
 *
 */

struct RTsk_buff *RTskb_copy(struct RTsk_buff *skb)
{
	struct RTsk_buff *n;
	pt_obj_list pt;	

	n = NULL;
	pt = get_obj_list_item(&RTskbuffs);
	if (pt!=NULL){
		n = (RTsk_buff*)pt->object;
		memcpy(n,skb,sizeof(struct RTsk_buff));
		n->data = n->bdata + (skb->data - skb->bdata);
		n->head = n->bdata + (skb->head - skb->bdata);;
		n->tail = n->bdata + (skb->tail - skb->bdata);
		n->end =  n->bdata + (skb->end - skb->bdata);
		n->obj_list = pt;
	}
	atomic_set(&n->users, 1);		
	atomic_set(RTskb_dataref(n), 1);
	n->cloned = 0;
	
	return n;
}

// *********************************************************************

//Init RTsk_buff's
int initRTskbuffs(int n_buffs){

	//This is our temporary 'slab' memory allocator! 
	if (init_obj_list(&RTskbuffs,n_buffs,sizeof(RTsk_buff))){
		return -ENOMEM;
	}

	return 0;
}

//Delete RTsk_buff memory buffers
void endRTskbuffs(void){
	free_obj_list(&RTskbuffs);
}



