#include <linux/config.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/if_ether.h>

#include "rtip/RTskbuff.h"
#include "rtip/RTdev.h"

#include <rtai_fifos.h>
#include <rtai_sched.h>
#include <rtai.h>

MODULE_AUTHOR ("Pedro Martins <pmartins@di.fc.ul.pt>");
MODULE_DESCRIPTION ("RTIP test app");

MODULE_PARM(display_period, "i");
MODULE_PARM(send_period, "i");

MODULE_PARM_DESC(display_period, "period for information display (in seconds)");
MODULE_PARM_DESC(send_period, "period for packet relay (in nanoseconds)");

int display_period = 5;
unsigned long long send_period = 10000000;
int omission_degree=1;

MODULE_LICENSE("GPL");

EXPORT_NO_SYMBOLS;

static RT_TASK tasks[3];

unsigned long long seq_number;

#define MESSAGE_SIZE 1000

typedef struct{
	unsigned long long delay;
	unsigned long long Osnd_time;
	unsigned long long mac_addr;
}rnd_info_t;

typedef struct {
	unsigned char magic_id;			//0xF1
	unsigned long long s_mac_addr;
	char message_text[20];	
	unsigned long long seq_number;
	unsigned long long snd_time;
	int n_indexes;
}msg_header_t;

#define N_MACHINES (MESSAGE_SIZE - sizeof(msg_header_t))/sizeof(rnd_info_t)

typedef struct {
	msg_header_t	hdr;
	rnd_info_t	rnd_info[N_MACHINES];
} message_t;

typedef struct {
	char 		  	ocup;
	unsigned long long	mac_addr;
	unsigned long long 	Osnd_time;
	unsigned long long 	rcv_time;	
	unsigned long long	lost;
	unsigned long long	duplicated_packs;
	unsigned long long 	seq;
	unsigned long long	max_rndtrip;
	unsigned long long	min_rndtrip;
	unsigned long long	max_rndtrip_since_last_display;
	unsigned long long	min_rndtrip_since_last_display;
	unsigned long long	rx_packets;
	char 			text[20];
} last_info_t;

unsigned int n_machines;
unsigned int n_indexes;

spinlock_t	info_display_lock;

message_t snd_message, rcv_message;
last_info_t last_info[N_MACHINES];

//=======================================================================

int getLastIndex(u64 macAddr,int * pt_free){
	int a;
	*pt_free = -1;
	
	for (a=0;a<n_machines;a++){
		if (last_info[a].ocup){
			if (last_info[a].mac_addr==macAddr)
				return a;
		}
			else
				if (*pt_free == -1)
					*pt_free = a;
	}
	return -1;
}

rnd_info_t * getMyRndInf(message_t* rcv_msg){
	int a;
	
	for (a=0;a<rcv_msg->hdr.n_indexes;a++){
		if (snd_message.hdr.s_mac_addr==rcv_msg->rnd_info[a].mac_addr)
			return &rcv_msg->rnd_info[a];
	}

	return NULL;
}

void process_rcv(message_t* rcv_msg, unsigned long long rcv_time){
	int index, pt_free;
	rnd_info_t *rndInfo;
	unsigned long long rndtrip;

	if (rcv_msg->hdr.magic_id!=0xF1)
		return;

	index = getLastIndex(rcv_msg->hdr.s_mac_addr,&pt_free);
	if (index == -1){	//not found
		if (pt_free!=-1){	//there exists a free index
			rt_spin_lock(&info_display_lock);
			index = pt_free;			
			last_info[index].seq = rcv_msg->hdr.seq_number;
			last_info[index].ocup = 1;
			last_info[index].rcv_time = rcv_time;
			last_info[index].Osnd_time = rcv_msg->hdr.snd_time;
			last_info[index].mac_addr = rcv_msg->hdr.s_mac_addr;
			last_info[index].duplicated_packs=0;
			memcpy(&last_info[index].text[0],&rcv_msg->hdr.message_text[0],20);
			last_info[index].rx_packets = 1;			
			n_indexes++;
			rt_spin_unlock(&info_display_lock);
		}
		return;
	}
	rt_spin_lock(&info_display_lock);
	
	if (last_info[index].seq==rcv_msg->hdr.seq_number){
		last_info[index].duplicated_packs++;
		rt_spin_unlock(&info_display_lock);	
		return;
	}

	last_info[index].rcv_time = rcv_time;
	last_info[index].Osnd_time = rcv_msg->hdr.snd_time;
	rt_spin_unlock(&info_display_lock);

	rndInfo = getMyRndInf(rcv_msg);

	if (rndInfo==NULL)
		return;
			

	rndtrip=rcv_time-rndInfo->Osnd_time-rndInfo->delay;

	rt_spin_lock(&info_display_lock);

	if ((last_info[index].max_rndtrip<rndtrip)||(last_info[index].max_rndtrip==0))
		last_info[index].max_rndtrip=rndtrip;
	if ((last_info[index].min_rndtrip>rndtrip)||(last_info[index].min_rndtrip==0))
		last_info[index].min_rndtrip=rndtrip;
	if ((last_info[index].min_rndtrip_since_last_display>rndtrip)||(last_info[index].min_rndtrip_since_last_display==0))
		last_info[index].min_rndtrip_since_last_display=rndtrip;
	if ((last_info[index].max_rndtrip_since_last_display<rndtrip)||(last_info[index].max_rndtrip_since_last_display==0))
		last_info[index].max_rndtrip_since_last_display=rndtrip;
	
	if (last_info[index].seq!=rcv_msg->hdr.seq_number-1)
		last_info[index].lost+=rcv_msg->hdr.seq_number-last_info[index].seq-1;
	
	last_info[index].rx_packets++;

	last_info[index].seq = rcv_msg->hdr.seq_number;

	rt_spin_unlock(&info_display_lock);	
}

int buildPacket(message_t* snd_msg){
	int a, pt=0;

	snd_msg->hdr.seq_number=seq_number;
	snd_msg->hdr.snd_time=rt_get_time_ns();	
	
	for (a=0;a<n_machines;a++){
		if (last_info[a].ocup){	
			snd_msg->rnd_info[pt].Osnd_time = last_info[a].Osnd_time;
			snd_msg->rnd_info[pt].delay = snd_msg->hdr.snd_time - last_info[a].rcv_time ;						
			snd_msg->rnd_info[pt].mac_addr = last_info[a].mac_addr;
			pt++;	
		}
	}

	snd_msg->hdr.n_indexes=pt;	

	return pt;
}

//=======================================================================

static void display_info(int arg){
	int a,b;

	rt_task_make_periodic_relative_ns(rt_whoami(), 0, (unsigned long long)display_period*1000000000);
	
	while (1) {
		rt_task_wait_period();	
		rt_spin_lock(&info_display_lock);
		for (a=0;a<n_indexes;a++){		
		if (last_info[a].ocup){	
			if (last_info[a].mac_addr==snd_message.hdr.s_mac_addr)
				rt_printk("from: LOCALHOST");
					else{
						rt_printk("from: ");
						rt_printk("%X:",last_info[a].mac_addr>>40);
						rt_printk("%X:",(last_info[a].mac_addr>>32)&0x000000FF);
						rt_printk("%X:",(last_info[a].mac_addr>>24)&0x000000FF);
						rt_printk("%X:",(last_info[a].mac_addr>>16)&0x000000FF);
						rt_printk("%X:",(last_info[a].mac_addr>>8)&0x000000FF);
						rt_printk("%X",(last_info[a].mac_addr)&0x000000FF);
						
					}
				rt_printk(", rx packets: %llu, lost packets: %llu rx duplicated: %lu",last_info[a].rx_packets,last_info[a].lost,last_info[a].duplicated_packs);			
				rt_printk(", max rndtrip: %llu, max rndtrip since last display: %llu",last_info[a].max_rndtrip,last_info[a].max_rndtrip_since_last_display);					
				rt_printk(", min rndtrip: %llu, min rndtrip since last display: %llu",last_info[a].min_rndtrip,last_info[a].min_rndtrip_since_last_display);								
				rt_printk(", message text: '%s'", last_info[a].text);							
			rt_printk("\n\n");
			last_info[a].min_rndtrip_since_last_display=0;
			last_info[a].max_rndtrip_since_last_display=0;
			
		}	
		}
		rt_spin_unlock(&info_display_lock);
	}
}

int ocounter=0;

static void thread_send(int arg)
{
	int rnd_fields=0;
	unsigned long long macaddr = RTgetMacAddr();
	int cnt;

	rt_task_make_periodic_relative_ns(rt_whoami(), 0, (unsigned long)send_period/(omission_degree+1));

	snd_message.hdr.s_mac_addr = macaddr;
	strcpy(snd_message.hdr.message_text,"RTIP ping message");
	snd_message.hdr.magic_id = 0xF1;
	
	while (1) {
		rt_task_wait_period();	
				
		rnd_fields = buildPacket(&snd_message);		
	//	for (cnt=0;cnt<omission_degree+1;cnt++)
		RTsendMessage((void*)&snd_message,sizeof(msg_header_t)+(sizeof(rnd_info_t)*rnd_fields));					
		if (ocounter==0){			
			seq_number++;
			ocounter++;
		}
		if (ocounter==omission_degree+1)
			ocounter=0;	
			 else
			  ocounter++;
	}
}

static void thread_receive(int arg)
{
	struct RTsk_buff* pt_sk;

	rt_task_make_periodic_relative_ns(rt_whoami(), 0, send_period>>1);	

	while (1) {
		rt_task_wait_period();
		do {
			pt_sk = RTreadMessage();					
			if (pt_sk!=NULL){
				memcpy((void*)&rcv_message,pt_sk->data+14,pt_sk->len);	
				process_rcv(&rcv_message,pt_sk->timestamp);
				RTreleaseMessage(pt_sk);		
			}
		} while (pt_sk!=NULL);
	}
}

static int test_init (void)
{
	rt_printk("This test can run with a maximum of %d machines.\n",N_MACHINES);
	rt_task_init(&tasks[0], thread_send, 0, 1000, 0, 0, 0);
	rt_task_init(&tasks[1], thread_receive, 0, 1000, 1, 0, 0);	
	rt_task_init(&tasks[2], display_info, 0, 1000, 1, 0, 0);	
	rt_set_oneshot_mode();
	start_rt_timer(0);
	seq_number = 0;
	n_machines = N_MACHINES; 
	n_indexes = 0;
	memset(&last_info,0,sizeof(last_info_t));
	spin_lock_init(&info_display_lock);
	RTstartReceive();	
	rt_task_resume(&tasks[0]);
	rt_task_resume(&tasks[1]);		
	rt_task_resume(&tasks[2]);		
	return 0;
}

static void  test_cleanup (void)
{
	stop_rt_timer();
	rt_task_delete(&tasks[0]);
	rt_task_delete(&tasks[1]);
	rt_task_delete(&tasks[2]);
	RTstopReceive();		
}

module_init(test_init);
module_exit(test_cleanup);

